local thisUtil = "BR146 MOD STRINGS v1.0"

--print("This is "..thisUtil)

local br146_mod_strings = {}
local br146_mod_strings_basic = {}

local lb = "\n"
local abs = lb..lb
local licence = abs.."Copyright © 2017 \"BR146\" from transportfever.net (User page: transportfever.net/index.php/User/18313)"..abs.."Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the \"Software\"), to deal in the Software without restriction, including the right to distribute and without limitation the rights to use, copy and/or modify the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:"..abs.."The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. Distribution permitted only at \"www.transportfever.net\"."..abs.."THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE."

br146_mod_strings_basic.lb = lb
br146_mod_strings_basic.abs = abs
br146_mod_strings_basic.licence = licence


local function handleTranslationTable(ident, translationTable, stringsLuaData)
	for lang, text in pairs(translationTable) do
		if(stringsLuaData[lang] == nil) then
			stringsLuaData[lang] = {}
		end
			
		--print(thisUtil, "WRITE", lang, ident)
		stringsLuaData[lang][ident] = text
	end
end

local function getNestedTranslation(ident, t, stringsLuaData)
	local isTranslationTable = true
	for k,v in pairs(t) do
		isTranslationTable = isTranslationTable and type(v) ~= "table"
	end
	
	if(isTranslationTable) then
		handleTranslationTable(ident, t, stringsLuaData)
	else
		for key,nextT in pairs(t) do
			getNestedTranslation(ident.."@"..key, nextT, stringsLuaData)
		end
	end
end

local function convertToStringsLuaData(modId, mystrings)
	local stringsLuaData = {}
	
	getNestedTranslation(modId, mystrings, stringsLuaData)
	
	return stringsLuaData
end

local function findModId(path)
	local modId = nil
	modId = modId or path:match("([a-zA-Z0-9]*_[^%/%\\]*_[0-9]*)[/\\]strings%.lua$")
	modId = modId or path:match("([a-zA-Z0-9]*_[^%/%\\]*_[0-9]*)[/\\]mod%.lua$")
	modId = modId or path:match("([a-zA-Z0-9]*_[^%/%\\]*_[0-9]*)[/\\]res[/\\].*$")
	
	return modId
end

local function createStringsLua(mystrings)
	local info = debug.getinfo(2,'S')
	if(info == nil) then
		print(thisUtil, "WARNING", "Failed to create strings.lua")
		return {}
	end
	
	local stringsLuaFile = info.source
	
	local modId = findModId(stringsLuaFile)
	
	--print(thisUtil, "WRITE", "caller file", modLuaFile)
	--print(thisUtil, "WRITE", "mod id", modId)
	if(modId==nil) then
		error("\nInvalid call in 'br146_mod_strings_v1_0.lua'.\n"..
			"Notify BR146 (https://www.transportfever.net/index.php/User/18313-BR146/), and provide following information:\n"..
			"\tFile: "..stringsLuaFile.."\n\n")
			
		return {}
	else
		return convertToStringsLuaData(modId, mystrings)
	end
end

local function get(key)
	local info = debug.getinfo(2,'S');
	if(info == nil) then
		print(thisUtil, "WARNING", "Failed to find mod-id...")
		return "NOT FOUND"
	end
	
	local modLuaFile = info.source
	
	local modId = findModId(modLuaFile)
		
	if(modId==nil) then
		error("\nInvalid call in 'br146_mod_strings_v1_0.lua'.\n"..
			"Notify BR146 (https://www.transportfever.net/index.php/User/18313-BR146/), and provide following information:\n"..
			"\tFile: "..modLuaFile.."\n\n")
			
		return "NOT FOUND"
	else
		--print(thisUtil, "REQUEST", modId.."@"..key)
		
		return _(modId.."@"..key)
	end
end

local function makeHook(k)
	local textT = {key = k}
	
	setmetatable(textT, {
		__index = function(str,nextKey)
			return makeHook(str.key.."@"..nextKey)
		end,
		
		__call = function(str)
			return get(str.key)
		end
	})
		
	return textT
end

setmetatable(br146_mod_strings, {
	__index = function (t, key)
		return makeHook(key)
	end,

	__call = function(t, mystrings)
		if(mystrings == nil) then
			return br146_mod_strings_basic
		else
			return createStringsLua(mystrings)
		end
	end
})

return br146_mod_strings