local thisUtil = "BR146 PARAMETER UTIL v1.1"

local parameterUtil = {}
parameterUtil.__index = parameterUtil
setmetatable(parameterUtil, {
	__call = function(util)
		return util.newInstance()
	end,
})

local function findModId(path)
	local modId = nil
	modId = modId or path:match("([a-zA-Z0-9]*_[^%/%\\]*_[0-9]*)[/\\]strings%.lua$")
	modId = modId or path:match("([a-zA-Z0-9]*_[^%/%\\]*_[0-9]*)[/\\]mod%.lua$")
	modId = modId or path:match("([a-zA-Z0-9]*_[^%/%\\]*_[0-9]*)[/\\]res[/\\].*$")
	
	return modId
end

function parameterUtil.newInstance()
	local instance = {}
	setmetatable(instance,parameterUtil)
	getmetatable(instance).__call = instance.map
	
	local info = debug.getinfo(2,'S')
	if(info == nil) then
		print(thisUtil, "WARNING", "Failed to find mod-id...")
		
		local prefix = {}
		for i = 1,20 do
			prefix[i] = string.char(math.random(1, 255))
		end
		instance.prefix = table.concat(prefix)
	else
		instance.prefix = findModId(info.source)
	end
	
	instance.parameter = {}
	
	instance.mapper = {}
		
	return instance
end

function parameterUtil:getParameter()
	return self.parameter
end

function parameterUtil:map(paramKey, params)
	if(params == nil) then
		params = paramKey
		
		local mapped = {}
		
		for key, mapFn in pairs(self.mapper) do
			mapped[key] = mapFn(params)
		end
		
		return mapped
	else
		return self.mapper[paramKey](params)
	end
end

function parameterUtil:makeUpdateFn(updateFn)
	return function(paramUnmapped)
		return updateFn(self:map(paramUnmapped), paramUnmapped)
	end
end

function parameterUtil:addParameter(pKey, pName, default, values)
	local p = {
		simpleKey = pKey,
		
		key = self.prefix .. pKey,
		name = _(pName),
		values = values,
		defaultIndex = default,
	}
	
	table.insert(self.parameter, p)
	self:doMapping(p)
end

function parameterUtil:addYesNoParam(pKey, pName, default, mapYes, mapNo)
	local state = default or false
	if(state) then
		defaultIndex = 0
	else
		defaultIndex = 1
	end
	
	local mapY = mapYes or 1
	local mapN = mapNo or 0

	local p = {
		simpleKey = pKey,
		
		key = self.prefix .. pKey,
		name = _(pName),
		values = {
			{ "ja", map = mapY },
			{ "nein", map = mapN }
		},
		defaultIndex = defaultIndex,
	}
	
	table.insert(self.parameter, p)
	self:doMapping(p)
end

function parameterUtil:doMapping(para)
	para.mapping = {}
		
	for index, value in pairs(para.values) do
		if(type(value) == "table") then
			para.mapping[index] = value.map
			para.values[index] = _("${v}") % { v = value[1] }
		else
			para.mapping[index] = value
			para.values[index] = _("${v}") % { v = value }
		end
	end
		
	self.mapper[para.simpleKey] = function (params) 
		if(params == nil) then
			return #para.mapping
		else
			local index = 1
		
			if(type(params) == "table") then
				index = params[para.key] + 1
			else
				index = params
			end
			
			return para.mapping[index]
		end
	end
end

return parameterUtil